#include "mex.h"
/*

	Solves poisson's equation to get the streamlines
	psi=mfish(vort, dx, dz, psi_top, psi_bot);
	Danny Chapman 19/09/2000

 */

void mfish(double *psi, double *vort, double dx, double dz, double
					 psi_top, double psi_bot, int rows, int cols);

void printpsi(double *psi, int i, int j, int rows) {
	int ii, jj;
	for (ii=i ; ii < i+5 ; ii++) {
		for (jj=j ; jj < j+5 ; jj++) {
			printf("%6.3f ",psi[ii+jj*rows]);
		}
		printf("\n");
	}
	printf("\n");
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double *psi, *vort;
	double dx, dz, psi_top, psi_bot;
	int rows, cols;

	/* Check arguments */
	if (nrhs!=5)
		mexErrMsgTxt("Three inputs required");
/* 	if (nlhs!=1) */
/* 		mexErrMsgTxt("One output required"); */
	
	/* Create a pointer to the input matrices, vu and Z */
	vort=mxGetPr(prhs[0]);
	rows=mxGetM(prhs[0]);
	cols=mxGetN(prhs[0]);

	/* Get the scalar input, dz */
	dx=mxGetScalar(prhs[1]);
	dz=mxGetScalar(prhs[2]);

	psi_top=mxGetScalar(prhs[3]);
	psi_bot=mxGetScalar(prhs[4]);
	
	/* Set the ouput pointer to the output matrix */
	plhs[0]=mxCreateDoubleMatrix(rows,cols, mxREAL);

	/* Create a pointer to a copy of the ouput matrix*/
	psi=mxGetPr(plhs[0]);

	/* Call the shear subroutine */
	mfish(psi,vort,dx,dz,psi_top,psi_bot,rows,cols);
}

void mfish(double *psi, double *vort, double dx, double dz, double
					 psi_top, double psi_bot, int rows, int cols)
{
	int i, j, count;
	double dx2, dz2, dt, dtdx2, dtdz2;

	dx2=dx*dx;
	dz2=dz*dz;
	dt = (dx2/4 <  dz2/4 ? dx2/4 : dz2/4);
	dtdx2=dt/dx2;
	dtdz2=dt/dz2;

	/* boundary conditions */
	for (i=0; i < rows ; i++) {
		psi[i+0*rows]=psi_bot;
		psi[i+(cols-1)*rows]=psi_top;
	}
	/*
left and right are going to be given by vertical velocity=0..., and
we will update psi at the boundary
*/

	for (count=0 ; count < 5000 ; count++) {
		/*		printf("count=%d\n",count); */
		/*    printpsi(psi, 0,0,rows); */
		for (i=1; i < rows-1 ; i++) {
			for (j=rows; j < (cols-1)*rows ; j+=rows) {
				psi[i+j]=psi[i+j]+
					dtdx2*(psi[i+1+j]+psi[i-1+j]-2*psi[i+j])
					+dtdz2*(psi[i+j+rows]+psi[i+j-rows]-2*psi[i+j])
					-dt*vort[i+j];
			}
		}

		for (j=rows; j < (cols-1)*rows ; j+=rows) {
			psi[0+j]=psi[0+j]+
				dtdx2*(psi[1+j]-psi[0+j])+
				dtdz2*(psi[0+j+rows]+psi[0+j-rows]-2*psi[0+j])
				-dt*vort[0+j];
			psi[rows-1+j]=psi[rows-1+j]
				+dtdx2*(-psi[rows-1+j]+psi[rows-2+j])
				+dtdz2*(psi[rows-1+j+rows]+psi[rows-1+j-rows]-2*psi[rows-1+j])
				-dt*vort[rows-1+j];
		}
	}
	/* that's it */
}




